`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Modified Date: 2020-02-28
// Design Name:
// Module Name: csr_mcycle
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////

module csr_mcycle
(
    input sys_clk,

    input i_dbg_mode,
    input i_dbg_stpcyl,

    input [ 11: 0 ] i_csr_addr,
    input [ 31: 0 ] i_csr_val,
    input i_csr_wen,

    output reg [ 31: 0 ] o_mcycle_l,
    output reg [ 31: 0 ] o_mcycle_h,
    
    input rst_n
);



wire dbg_stop = i_dbg_mode & i_dbg_stpcyl;

always@( posedge sys_clk or negedge rst_n )
if ( !rst_n )
begin
    o_mcycle_l <= 32'b0;
    o_mcycle_h <= 32'b0;
end
else if ( i_csr_wen )
begin
    case ( i_csr_addr )
        12'hb00: o_mcycle_l <= i_csr_val;
        12'hb80: o_mcycle_h <= i_csr_val;
        default: ;
    endcase
end
else
begin
    if ( !dbg_stop )
    begin
        if ( o_mcycle_l == 32'hffff_ffff )
        begin
            o_mcycle_h <= o_mcycle_h + 1;
            o_mcycle_l <= 0;
        end
        else
            o_mcycle_l <= o_mcycle_l + 1;
    end
end




endmodule
