`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Modified Date: 2020-02-28
// Design Name:
// Module Name: csr_mepc
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////


module csr_mepc
(
    input           sys_clk,
    
    input           i_irq_src,
    input           i_exp_src,
    input [ 31: 0 ] i_exe_pc,
    
    input           i_acc_dis,
    input [ 11: 0 ] i_csr_addr,
    input [ 31: 0 ] i_csr_val,
    input           i_csr_wen,
    
    output [ 31: 0 ] o_mepc,

    input           rst_n
);
/*
mepc : Machine Exception Program Counter   RW
*/

wire [ 31: 0 ]  i_mepc;
wire [ 31: 0 ]  mepc_r;
/*
assign i_mepc = i_irq_src ? ( i_exe_pc + 4 ) :
                      ( i_exp_src ? i_exe_pc : mepc_r );
*/

wire mepc_valid = (i_irq_src | i_exp_src) ? 1'b1 : 1'b0;
assign i_mepc = mepc_valid ? i_exe_pc : mepc_r;

wire wbck_csr_wen = i_csr_wen & ( ~i_acc_dis );

//0x341 MRW mepc Machine exception program counter.
wire sel_mepc = ( i_csr_addr == 12'h341 );
wire wr_mepc = sel_mepc & wbck_csr_wen;
wire mepc_ena = wr_mepc | mepc_valid;

wire [ 31: 0 ] mepc_nxt;
assign mepc_nxt[ 31: 1 ] = mepc_valid ? i_mepc[ 31 : 1 ] : i_csr_val[ 31 : 1 ];
assign mepc_nxt[ 0 ] = 1'b0; // have to be 1'b0, otherwise  will generate the misalign exception according to ISA

fii_dfflr #( 32 ) epc_dfflr ( mepc_ena, mepc_nxt, mepc_r, sys_clk, rst_n );
assign o_mepc = mepc_r;

endmodule
