`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Modified Date: 2020-02-28
// Design Name:
// Module Name: csr_minstret
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////

module csr_minstret     //minstret   -----machine  instruction retire
(
    input sys_clk,

    input i_EXE_vld,

    input i_dbg_mode,
    input i_dbg_stpcyl,

    input [ 11: 0 ] i_csr_addr,
    input [ 31: 0 ] i_csr_val,
    input i_csr_wen,
    
    output reg [ 31: 0 ] o_minstret_l,
    output reg [ 31: 0 ] o_minstret_h,

    input rst_n
);

wire dbg_stop = i_dbg_mode & i_dbg_stpcyl;
always@( posedge sys_clk or negedge rst_n )
if ( !rst_n )
begin
    o_minstret_l <= 32'b0;
    o_minstret_h <= 32'b0;
end
else if ( i_csr_wen )
begin
    case ( i_csr_addr )
        12'hb02: o_minstret_l <= i_csr_val;
        12'hb82: o_minstret_h <= i_csr_val;
        default: ;
    endcase
end
else if ( ( i_EXE_vld ) && ( dbg_stop == 1'b0 ) )
begin
    if ( o_minstret_l == 32'hffff_ffff )
    begin
        o_minstret_l <= 0;
        o_minstret_h <= o_minstret_h + 1;
    end
    else
        o_minstret_l <= o_minstret_l + 1;
end
    
endmodule
