`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Modified Date: 2020-02-28
// Design Name:
// Module Name: csr_mtval
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////


module csr_mtval
(
    input sys_clk,

    input i_irq_src,
    input i_exp_src,
    input [ 31: 0 ] i_exe_pc,
    input [ 31: 0 ] i_ir,

    input [ 11: 0 ] i_csr_addr,
    input [ 31: 0 ] i_csr_val,
    input i_csr_wen,
    
    output [ 31: 0 ] o_mtval,

    input rst_n
);

/*
mtval  Machine Trap Value Register    RW
*/

wire sel_mtval = ( i_csr_addr == 12'h343 );

wire trap_mtval_ena = sel_mtval & i_exp_src;
wire [31:0] i_trap_mtval_val;

//0x343 MRW o_mtval Machine bad address.
wire wr_mtval = sel_mtval & i_csr_wen;

wire mtval_ena = ( wr_mtval & i_csr_wen ) | trap_mtval_ena;
wire [ 31: 0 ] mtval_r;
wire [ 31: 0 ] mtval_nxt;
assign mtval_nxt = trap_mtval_ena ? i_trap_mtval_val : i_csr_val;
fii_dfflr #( 32 ) mtval_dfflr ( mtval_ena, mtval_nxt, mtval_r, sys_clk, rst_n );

assign o_mtval = mtval_r;

endmodule
