`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Modified Date: 2020-02-28
// Design Name:
// Module Name: csr_mtvec
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////


module csr_mtvec
(
    input sys_clk,

    input i_acc_dis,

    input [ 11: 0 ] i_csr_addr,
    input [ 31: 0 ] i_csr_val,
    input i_csr_wen,

    output [ 31: 0 ] o_mtvec,

    input rst_n
);
/*
o_mtvec: Machine Trap-Vector Base-Address Register  WR

MXLEN-1                  2   1 0
--------------------------------------------------
BASE[MXLEN-1:2] (WARL )   |  MODE (WAR L )
--------------------------------------------------
MXLEN-2                      2


MODE: Value Name Description
0 Direct All exceptions set pc to BASE.
1 Vectored Asynchronous interrupts set pc to BASE+4×cause.
≥2 — Reserved

*/

wire wbck_csr_wen = i_csr_wen & ( ~i_acc_dis );
//0x305 MRW o_mtvec Machine trap-handler base address.
wire sel_mtvec = ( i_csr_addr == 12'h305 );

wire wr_mtvec = sel_mtvec & i_csr_wen;
wire mtvec_ena = ( wr_mtvec & wbck_csr_wen );
wire [ 31: 0 ] mtvec_r;
wire [ 31: 0 ] mtvec_nxt = { i_csr_val[ 31: 2 ] , 2'b00 }; //Direct All exceptions set pc to BASE.

fii_dfflr #( 32 ) mtvec_dfflr ( mtvec_ena, mtvec_nxt, mtvec_r, sys_clk, rst_n );

assign o_mtvec = mtvec_r;




endmodule
