`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Design Name:
// Module Name: D_sram
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1
// Description:
//
// Dependencies:
// Memory for DTCM
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////
//`define SIM
module D_sram
#( 
    parameter MEM_D_DEEP    = 1024, //memory data depth
    parameter MEM_D_W       = 32,   //memory data width
    parameter MEM_MSK_W     = 4,    //memory data mask width
    parameter MEM_ADDR_W    = 32    //memory address width
)
(
    input  clk,                             // ϵͳʱ 
    
    input  [ MEM_D_W - 1 : 0 ] din,         // д
    input  [ MEM_ADDR_W - 1 : 0 ] addr,     // ߵַ
    output [ MEM_D_W - 1: 0 ] dout,         // ݶ
    
    input  cs,                              // ģƬѡ
    input  we,                              // дź    
    input  [ MEM_MSK_W - 1: 0 ] wem,        // дλ   

    input  rst_n
);

//===============================================================================
reg  [ MEM_D_W - 1: 0 ] mem_r[ 0: MEM_D_DEEP - 1 ];
//===============================================================================
wire ren = cs & ( ~we );
wire [ MEM_MSK_W - 1: 0 ] wen = ( { MEM_MSK_W{ cs & we } } & wem );

reg  [ MEM_ADDR_W - 1: 0 ] addr_r = 0;
always @( posedge clk )
    addr_r <= addr;

wire [ MEM_ADDR_W - 1: 0 ] addr_mem = {addr[31:2]};
    
// integer                 mem_init_addr;
//===============================================================================
genvar wi;
generate
    for ( wi = 0; wi < MEM_MSK_W; wi = wi + 1 )
    begin: mem_write
        always @( posedge clk )
        begin
            if ( wen[ wi ] )
            begin
                mem_r[ addr_mem ][ 8 * wi + 7: 8 * wi ] <= din[ 8 * wi + 7: 8 * wi ];
            end
        end
    end
endgenerate
//===============================================================================
wire [ MEM_D_W - 1: 0 ] dout_pre;
wire [ MEM_D_W - 1: 0 ] dout_w;
//assign dout_pre = mem_r[ addr_r ];
assign dout_pre = mem_r[ addr_mem ];
assign dout_w = dout_pre;

/*
genvar ri;
generate
    for ( ri = 0; ri < MEM_D_W; ri = ri + 1 )
    begin: mem_read
`ifdef SIM//{
        assign dout_w[ ri ] = ( dout_pre[ ri ] === 1'bx ) ? 1'b0 : dout_pre[ ri ];
`else //}{
        assign dout_w[ ri ] = dout_pre[ ri ];
`endif//}
    end
endgenerate
*/
//===============================================================================
//wire [4:0] data_sft = addr[1:0] * 8;
wire [4:0] data_sft = {addr[1:0], 3'b000};
assign dout = dout_w >> data_sft;

//===============================================================================

endmodule
