`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Design Name:
// Module Name: fii_rv32i_core
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////


module fii_rv32i_core 
# (
    parameter [31:0] TMR_BASEADDR  = 32'h0200_0000,
    parameter [31:0] PLIC_BASEADDR = 32'h0c00_0000,
    parameter [31:0] CPU_BASEADDR  = 32'h8000_0000,
    parameter [31:0] MEM_BASEADDR  = 32'h9000_0000,
    parameter [31:0] UART_BASEADDR = 32'he000_0000,
    parameter [31:0] GPIO_BASEADDR = 32'hf000_0000
)
(
    input  sys_clk,                 // ϵͳʱ
    
//====================================
    input  [ 31: 0 ] i_GPIO_dina,   // gpio A  д gpio
    output [ 31: 0 ] o_GPIO_douta,  // gpio A   gpio  
    output [ 31: 0 ] o_GPIO_ta,     // gpio A  ѡ
    
    input  [ 31: 0 ] i_GPIO_dinb,   // gpio B  д gpio
    output [ 31: 0 ] o_GPIO_doutb,  // gpio B   gpio  
    output [ 31: 0 ] o_GPIO_tb,     // gpio B  ѡ
    
    input  [ 31: 0 ] i_GPIO_dinc,   // gpio C  д gpio
    output [ 31: 0 ] o_GPIO_doutc,  // gpio C   gpio  
    output [ 31: 0 ] o_GPIO_tc,     // gpio C  ѡ

    input  [ 31: 0 ] i_GPIO_dind,   // gpio D  д gpio
    output [ 31: 0 ] o_GPIO_doutd,  // gpio D   gpio  
    output [ 31: 0 ] o_GPIO_td,     // gpio D  ѡ

    output           txd_start,     // ֪ͨ uart phy ģ һbyte enable
    output [7:0]     txd_data,      // ֪ͨ uart phy ģ   byte
    input            txd_done,      // uart phy  
    
//====================================
    output  [31:0]  o_sft_int_v,    // жϼĴ
    output  [31:0]  o_timer_l,      // timer ʱ Ĵ 32λ
    output  [31:0]  o_timer_h,      // timer ʱ Ĵ 32λ
            
    input   [31:0]  i_timer_l,      // ȡǰ ʱֵ 32λ
    input   [31:0]  i_timer_h,      // ȡǰ ʱֵ 32λ
            
    output  [31:0]  o_tcmp_l,       // timer ȽϼĴ 32λ
    output  [31:0]  o_tcmp_h,       // timer ȽϼĴ 32λ
            
    output  [1:0]   o_timer_valid,  // timer Ч
    output  [31:0]  o_tm_ctrl,      // timer ƼĴ
             
    
    input   [31:0]  code_addr,      // ͨ uart    ַӿ
    input   [31:0]  code_din,       // ͨ uart    ݽӿ
    input           code_wea,       // ͨ uart    д enable
    
//====================================
    input           i_ext_irq,      // ⲿģ  ⲿж
    input           i_sft_irq,      // ģ  ж
    input           i_tmr_irq,      // ʱģ  ʱж

    output          o_meie,         // CSR Ĵ  machine ⲿж enable
    output          o_msie,         // CSR Ĵ  machine ж enable
    output          o_mtie,         // CSR Ĵ  machine ʱж enable
    output          o_glb_irq,      // CSR Ĵ  machine ȫж enable

//====================================
    input           i_cpu_reset,
    input           rst_n
);

//===============================================================================
localparam [ 2: 0 ] IDLE  = 8'd0,
                    I_FCH = 8'd1,
                    I_EXE = 8'd2,
                    I_LS  = 8'd3,
                    I_WB  = 8'd4,
                    I_RO1 = 8'd5,
                    I_RO2 = 8'd6,
                    I_RO3 = 8'd7;




wire ls_need;



// load program data
(* mark_debug = "yes" *)wire o_CPU_cs;
wire [ 31: 0 ] o_CPU_PC;



//===============================================================================
(* mark_debug = "yes" *) reg [ 2: 0 ] instr_st = 0;
reg [ 2: 0 ] instr_st_nxt = 0;
always@( posedge sys_clk )
if (( rst_n == 1'b0 ) | i_cpu_reset )     instr_st <= IDLE;
else    instr_st <= instr_st_nxt;



always @ ( * )
begin
    case ( instr_st )
    IDLE:   // 0
    begin
        if(!i_cpu_reset)
            instr_st_nxt = I_FCH;
        else
            instr_st_nxt = IDLE;
    end
    I_FCH:   // 1
    begin
        if(i_cpu_reset) instr_st_nxt = IDLE;
        else instr_st_nxt = I_EXE;
    end
    I_EXE:   // 2
    begin
        if ( ls_need & o_CPU_cs)
            instr_st_nxt = I_RO1;
        else 
            instr_st_nxt = I_WB;
    end
    I_WB:   // 4
    begin
        instr_st_nxt = I_FCH;
    end
    I_RO1:   // 5
    begin
        instr_st_nxt = I_RO2;
    end
    I_RO2:   // 6
    begin
        instr_st_nxt = I_FCH;
    end
    I_RO3:   // 7
    begin
        instr_st_nxt = I_FCH;
    end
    default : instr_st_nxt = IDLE;
    endcase
end
//===============================================================================
/*
reg [2:0] pc_load_r = 0;
always @ (posedge sys_clk or negedge rst_n)
if(~rst_n) pc_load_r <= 0;
else pc_load_r <= {pc_load_r[1:0], o_CPU_cs};
*/

(* mark_debug = "yes" *)reg            irq_fch_r = 0;
always @ (posedge sys_clk or negedge rst_n)
if(~rst_n) irq_fch_r <= 0;
else if(instr_st == I_FCH)  irq_fch_r <= (i_ext_irq | i_sft_irq | i_tmr_irq);


//(* mark_debug = "yes" *)wire w_irq_src = ({pc_load_r, o_CPU_cs} != 0) ? 1'b0 : irq_fch_r;
(* mark_debug = "yes" *)wire w_irq_src = o_CPU_cs ? 1'b0 : irq_fch_r;
(* mark_debug = "yes" *)wire jump_irq_pc = w_irq_src & o_glb_irq;
wire            w_exp_src = 0;
wire irq_exp = w_irq_src | w_exp_src ;

wire            mret;
wire    [31:0]  mepc;
wire    [31:0]  w_irq_pc;
(* mark_debug = "yes" *) reg [ 31: 0 ] i_fch_PC = 0;
reg [ 31: 0 ] res_PC = 0;
wire [31:0] w_exe_PC;  

always @( posedge sys_clk or negedge rst_n )
if ( !rst_n ) i_fch_PC <= CPU_BASEADDR;
else 
begin
    if ( instr_st == IDLE )         //0
        i_fch_PC <= CPU_BASEADDR;
    else if ( instr_st == I_EXE )   //2
    begin
        if( mret )
            i_fch_PC <= mepc;
        else if (jump_irq_pc)
            i_fch_PC <= w_irq_pc;
        else if( o_CPU_cs )
        begin
            i_fch_PC <= o_CPU_PC;
            res_PC <= w_exe_PC;
        end
        else i_fch_PC <= w_exe_PC;
    end
    else if ( instr_st == I_RO1 )
    begin
        i_fch_PC <= res_PC;
    end
end

//===============================================================================
(* mark_debug = "yes" *) reg EXE_vld = 0;
always @( posedge sys_clk or negedge rst_n )
if ( !rst_n ) EXE_vld <= 0;
else
begin
    if ( instr_st == I_FCH )        //1
        EXE_vld <= 1'b1;
    else
        EXE_vld <= 1'b0;
end


wire [ 31: 0 ] instr;

(* mark_debug = "yes" *)reg [ 31: 0 ] exe_instr = 0;
always @( posedge sys_clk or negedge rst_n )
if ( !rst_n ) exe_instr <= 0;
else if ( instr_st == I_FCH )
    exe_instr <= instr;



(* mark_debug = "true" *)reg i_CPU_load_vld = 0;
(* mark_debug = "true" *)reg [ 31: 0 ] i_CPU_load_data = 0;
always @( posedge sys_clk or negedge rst_n )
if ( !rst_n ) 
begin
    i_CPU_load_data <= 0;
    i_CPU_load_vld <= 0;
end
else if ( instr_st == I_RO2 )
begin
    i_CPU_load_data <= instr;
    i_CPU_load_vld <= 1'b1;
end
else  i_CPU_load_vld <= 1'b0;

//===============================================================================

(* mark_debug = "yes" *) wire [ 31: 0 ] instr_PC = i_fch_PC;

rv32I_exu  #
(
    .TMR_BASEADDR   ( TMR_BASEADDR ),
    .PLIC_BASEADDR  ( PLIC_BASEADDR ),
    .CPU_BASEADDR   ( CPU_BASEADDR ),
    .MEM_BASEADDR   ( MEM_BASEADDR ),
    .UART_BASEADDR  ( UART_BASEADDR ),
    .GPIO_BASEADDR  ( GPIO_BASEADDR )
)
rv32I_exu_inst
(
    .sys_clk        ( sys_clk ),
    
    .i_ir           ( exe_instr ),  // The instruction register
    .i_PC           ( instr_PC ),   // The PC register along with
    .i_EXE_vld      ( EXE_vld ),
    
    .i_CPU_load_vld ( i_CPU_load_vld),
    .i_CPU_load_data( i_CPU_load_data ),
    
    .o_ls_need      ( ls_need ),
    
    .o_exe_PC       ( w_exe_PC ),

    // load program data
    .o_CPU_cs       ( o_CPU_cs ),
    .o_CPU_PC       ( o_CPU_PC ),
    
    .i_ext_irq      ( i_ext_irq ),
    .i_sft_irq      ( i_sft_irq ),
    .i_tmr_irq      ( i_tmr_irq ),
    
    .o_meie         ( o_meie ),
    .o_msie         ( o_msie ),
    .o_mtie         ( o_mtie ),
    .o_glb_irq      ( o_glb_irq ),

    .i_irq_src      ( w_irq_src ),
    .i_exp_src      ( w_exp_src ),

    .o_mret         ( mret ),
    .o_irq_pc       ( w_irq_pc ),
    .o_mepc         ( mepc ),

//===============================================================================
    .i_GPIO_dina    ( i_GPIO_dina ),
    .o_GPIO_douta   ( o_GPIO_douta ),
    .o_GPIO_ta      ( o_GPIO_ta ),
    
    .i_GPIO_dinb    ( i_GPIO_dinb ),
    .o_GPIO_doutb   ( o_GPIO_doutb ),
    .o_GPIO_tb      ( o_GPIO_tb ),
    
    .i_GPIO_dinc    ( i_GPIO_dinc ),
    .o_GPIO_doutc   ( o_GPIO_doutc ),
    .o_GPIO_tc      ( o_GPIO_tc ),

    .i_GPIO_dind    ( i_GPIO_dind ),
    .o_GPIO_doutd   ( o_GPIO_doutd ),
    .o_GPIO_td      ( o_GPIO_td ),
    
    .txd_start      ( txd_start ),
    .txd_data       ( txd_data ),
    .txd_done       ( txd_done ),
    
    .o_sft_int_v    ( o_sft_int_v ),
    .i_timer_l      ( i_timer_l ),
    .i_timer_h      ( i_timer_h ),
                    
    .o_timer_l      ( o_timer_l ),
    .o_timer_h      ( o_timer_h ),
                    
    .o_tcmp_l       ( o_tcmp_l ),
    .o_tcmp_h       ( o_tcmp_h ),
                   
    .o_timer_valid  ( o_timer_valid ),
    .o_tm_ctrl      ( o_tm_ctrl),
    
    .i_cpu_reset    ( i_cpu_reset),
    .rst_n          ( rst_n )
);
//===============================================================================



//===============================================================================

wire instr_ena = ( instr_PC[ 31: 16 ] == CPU_BASEADDR[31:16] ) ? 1'b1 : 1'b0;

TDP_RAM_INSTR  program_inst
(
    .clka   ( sys_clk ),
    .ena    ( 1'b1 ),
    .wea    ( code_wea ),
    .addra  ( code_addr[31:2] ),
    .dina   ( code_din ),
    .douta  ( ),

    .clkb   ( sys_clk ),
    .enb    ( instr_ena ),
    .web    ( 1'b0 ),
    .addrb  ( instr_PC[ 31: 2 ] ), //8K 32bits, 32K byte
    .dinb   ( 32'b0 ),
    .doutb  ( instr )
);
//===============================================================================


endmodule
