
`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Design Name:
// Module Name: fii_GPIO
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////
//`define SIM

module fii_GPIO 
#(
    parameter GPIO_DEEP     = 8,    //memory data width
    parameter GPIO_W        = 32,   //memory data width
    parameter GPIO_MSK_W    = 4,    //memory data mask width
    parameter GPIO_ADDR_W   = 32    //memory address width
)
(
    input  clk,                                 // ϵͳʱ
    input  rst_n,
    
    input  [ GPIO_W - 1 : 0 ] i_ls_GPIO_din,    // cpu д
    output [ GPIO_W - 1 : 0 ] o_rb_GPIO_dout,   // cpu 
    
    input  [ GPIO_ADDR_W - 1 : 0 ] i_addr,      // ߵַ
    input  i_cs,                                // gpio ģ Ƭѡ
    input  i_we,                                // дź
    input  [ GPIO_MSK_W - 1: 0 ] i_wem,         // дź
// t = 0, output;  t = 1, input
    input  [ 31: 0 ] i_GPIO_dina,               // gpio ģa cpuд
    output [ 31: 0 ] o_GPIO_douta,              // gpio ģa cpu
    output [ 31: 0 ] o_GPIO_ta,                 // gpio ģa 
                                                
    input  [ 31: 0 ] i_GPIO_dinb,               // gpio ģb cpuд
    output [ 31: 0 ] o_GPIO_doutb,              // gpio ģb cpu
    output [ 31: 0 ] o_GPIO_tb,                 // gpio ģb 
                                                
    input  [ 31: 0 ] i_GPIO_dinc,               // gpio ģc cpuд
    output [ 31: 0 ] o_GPIO_doutc,              // gpio ģc cpu
    output [ 31: 0 ] o_GPIO_tc,                 // gpio ģc 
                                                
    input  [ 31: 0 ] i_GPIO_dind,               // gpio ģd cpuд
    output [ 31: 0 ] o_GPIO_doutd,              // gpio ģd cpu
    output [ 31: 0 ] o_GPIO_td                  // gpio ģd 
);
//===============================================================================
reg  [ GPIO_W - 1: 0 ] GPIO_r [ 0: GPIO_DEEP - 1 ];
wire [ GPIO_MSK_W - 1: 0 ] wen;
wire ren;
wire [ 2: 0 ] w_addr = i_addr[ 4: 2 ];

//===============================================================================
assign ren = i_cs & ( ~i_we );
assign wen = ( { GPIO_MSK_W{ i_cs & i_we } } & i_wem );

reg  [ GPIO_ADDR_W - 1: 0 ] addr_r = 0;
always@( posedge clk )
if ( ren )
    addr_r <= i_addr;

//===============================================================================
reg [7:0] i = 0;
initial
begin
    for ( i = 0; i < GPIO_DEEP; i = i + 1 )
    begin:IO_GPIO_INIT
        GPIO_r[i] <= ~0;
    end
end

genvar wi;
generate
    for ( wi = 0; wi < GPIO_MSK_W; wi = wi + 1 )
    begin:IO_write
        always @( posedge clk )
        begin
            if ( wen[ wi ] )
            begin
                GPIO_r[ w_addr ][ 8 * wi + 7: 8 * wi ] <= i_ls_GPIO_din[ 8 * wi + 7: 8 * wi ];
            end
        end
    end
endgenerate
//===============================================================================
wire [ GPIO_W - 1: 0 ] dout_pre;
wire [ GPIO_W - 1: 0 ] t_pre;
wire [ GPIO_W - 1: 0 ] reg_dout;
wire [ GPIO_W - 1: 0 ] in_dout[ 0: GPIO_DEEP / 2 - 1 ];

assign in_dout[ 0 ] = i_GPIO_dina;
assign in_dout[ 1 ] = i_GPIO_dinb;
assign in_dout[ 2 ] = i_GPIO_dinc;
assign in_dout[ 3 ] = i_GPIO_dind;

//wire [ 31: 0 ] in_dout_tmp = in_dout[ i_addr[ GPIO_DEEP / 2 - 1: 1 ] ];
wire [ 31: 0 ] in_dout_tmp = in_dout[ w_addr[ 2: 1 ]];


//assign t_pre = GPIO_r[ { addr_r[ GPIO_ADDR_W - 1: 1 ], 1'b1 } ];
//assign dout_pre  = GPIO_r[ { addr_r[ GPIO_ADDR_W - 1: 1 ], 1'b0 } ];
assign t_pre = GPIO_r[ { w_addr[ 2: 1 ], 1'b1 } ];
assign dout_pre  = GPIO_r[ { w_addr[ 2: 1 ], 1'b0 } ];

//===============================================================================
genvar rbi;
generate
    for ( rbi = 0; rbi < GPIO_W; rbi = rbi + 1 )
    begin:GPIO_readb
//                                                        input reg            output reg
        assign o_rb_GPIO_dout[ rbi ] = t_pre[ rbi ] ? in_dout_tmp[ rbi ] : dout_pre[ rbi ];
    end
endgenerate
//===============================================================================
assign o_GPIO_douta = GPIO_r[ 0 ];
assign o_GPIO_ta    = GPIO_r[ 1 ];

assign o_GPIO_doutb = GPIO_r[ 2 ];
assign o_GPIO_tb    = GPIO_r[ 3 ];

assign o_GPIO_doutc = GPIO_r[ 4 ];
assign o_GPIO_tc    = GPIO_r[ 5 ];

assign o_GPIO_doutd = GPIO_r[ 6 ];
assign o_GPIO_td    = GPIO_r[ 7 ];
//===============================================================================

endmodule
