`timescale 1ns / 1ps 
//////////////////////////////////////////////////////////////////////////////////
// Company:     Fraser Innavotion Inc
// Engineer:    Qianqi(Tim) Zhuang & William Gou
//
// Create Date: 09/25/2018 09:28:18 AM
// Design Name:
// Module Name: exu_BRANCH
// Project Name: RISC-V SOC RTL
// Target Devices: FII-PE7030, FII-PRX100T, FII-BM7100, FII-PRA040, FII-PRA006
// Tool Versions: vivado 18.1/18.2
// Description:
//
// Dependencies:
// this is a simple version of risc-v integer version,aim to show up basic concept of RTL
// Revision:1.0
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////
module exu_BRANCH
(
    input  sys_clk,                 // ϵͳʱ
    input  rst_n,

    input  i_EXE_vld,               // ִenable
    input  i_BRANCH,                // ģṩ branch ָ
    input  [ 5: 0 ] i_branch_instr, // branch ָ{rv32i_bgeu,rv32i_bge,rv32i_bltu,rv32i_blt,rv32i_bne,rv32i_beq};

    input  [ 31: 0 ] i_rs1_val,     // rs1 ͨüĴ
    input  [ 31: 0 ] i_rs2_val,     // rs2 ͨüĴ

    input  [ 31: 0 ] i_PC,          // ǰָpc
    input  [ 31: 0 ] i_B_imm,       // b-type 
    
    output reg o_B_vld,             // ת pc Ч branch valid
    output [ 31: 0 ] o_PC           // ׼תpc ֵ

) ;


wire [ 31: 0 ] op1 = i_rs1_val;
wire [ 31: 0 ] op2 = i_rs2_val;

wire [ 31: 0 ] op_xor = op1 ^ op2;
wire bit_or = |op_xor;

//                             bge                 blt
wire [ 33: 0 ] ext_op1 = ({i_branch_instr[4],i_branch_instr[2]} != 0 ) ? {op1[31],op1[31],op1} : {1'b0,1'b0,op1};
//                             bge                 blt
wire [ 33: 0 ] ext_op2 = ({i_branch_instr[4],i_branch_instr[2]} != 0 ) ? {op2[31],op2[31],op2} : {1'b0,1'b0,op2};
wire [ 33: 0 ] comp_ext_op2 = ~ext_op2 + 1;

wire [ 33: 0 ] op_sub = ext_op1 + comp_ext_op2; //ext_op1 - ext_op2;

assign o_PC = i_PC + i_B_imm;

//assign o_branch_instr = { rv32i_bgtu, rv32i_bgt, rv32i_bltu, rv32i_blt, rv32i_bne, rv32i_beq };
always @( * )
begin
    o_B_vld <= 1'b0;

    if ( i_EXE_vld & i_BRANCH )
    begin
        case ( i_branch_instr )
        6'b00_0001:
        begin //eq
            if ( !bit_or )
                o_B_vld <= 1'b1;
        end
        6'b00_0010:
        begin //bne
            if ( bit_or )
                o_B_vld <= 1'b1;
        end
        6'b00_0100:
        begin //blt
            if ( op_sub[ 33 ] )
                o_B_vld <= 1'b1;
        end
        6'b00_1000:
        begin //bltu
            if ( op_sub[ 33 ] )
                o_B_vld <= 1'b1;
        end
        6'b01_0000:
        begin //bge
            if ( !op_sub[ 33 ] )
                o_B_vld <= 1'b1;
        end
        6'b10_0000:
        begin //bgeu
            if ( !op_sub[ 33 ] )
                o_B_vld <= 1'b1;
        end
        default: ;
        endcase
    end
end

endmodule
