`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:04:09 07/24/2008 
// Design Name:  WilliamG
// Module Name:    pc_uart 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

    
module pc_uart
#(
    parameter SIM_DEBUG = "FALSE",
    parameter MAIN_CLK = 50
)
(
    input  clk,
    
    input  rxd,
    output txd,

    output reg code_wea = 0,
    output reg [31:0] code_addr = 0,
    output reg [31:0] code_din = 0,

    input  txd_start,
    input  [7:0] txd_data,
    output txd_done,
    
    output rxd_done,
    
    input  reset
);
//=======================================================

wire rxd_rdy;
wire [7:0] rxd_data;
wire rxd_eop;



wire [15:0] baudtick = (SIM_DEBUG == "TRUE") ? 16 : (MAIN_CLK == 100) ? 109 : (MAIN_CLK == 80) ? 87 : 54;
uart_transceiver  uart_transceiver_inst
(
    .sys_clk    (clk), // 100m

    .uart_rx    (rxd),
    .uart_tx    (txd),

    .divisor    (baudtick),  // 115200 * 8 

    .rx_data    (rxd_data),
    .rx_done    (rxd_rdy),
    .rx_eop     (rxd_eop),

    .tx_data    (txd_data),
    .tx_wr      (txd_start),
    .tx_done    (txd_done),
    .tx_busy    (),

    .test_pin   (),

    .sys_rst    (reset)
);

//=======================================================

reg [3:0] rx_st = 0;
always @ (posedge clk)
if(reset | rxd_eop)
begin
    if(rxd_eop) 
        code_wea <= 1;
        
    rx_st <= 0;
end 
else case (rx_st)
0:
begin
    code_wea <= 0;
    code_addr <= 0;
    code_din <= 0;
    rx_st <= 1;
end
1:
begin
    if(rxd_rdy) 
    begin
        code_din <= {rxd_data, code_din[31:08]};
        rx_st <= 2;
    end
end
2:
begin
    if(rxd_rdy) 
    begin
        code_din <= {rxd_data, code_din[31:08]};
        rx_st <= 3;
    end
end
3:
begin
    if(rxd_rdy) 
    begin
        code_din <= {rxd_data, code_din[31:08]};
        rx_st <= 4;
    end
end
4:
begin
    if(rxd_rdy) 
    begin
        code_din <= {rxd_data, code_din[31:08]};
        code_wea <= 1;
        rx_st <= 5;
    end
end
5:
begin
    code_wea <= 0;
    rx_st <= 6;
end
6:
begin
    code_din <= 0;
    code_addr <= code_addr + 4;
    rx_st <= 1; 
end
default : rx_st <= 0;
endcase

//=======================================================

assign rxd_done = rxd_eop;

//=======================================================

endmodule


